package cypher;

/* part of the Cypher program, a java client for babel.
 *
 * this file deals with stuff coming in from the server
 *
 * functions in this file are as follows:
 *
 *   -  check_server(DataInputStream in) 
 *   -  dataReady(DataHeader hed)
 *   -  doPingReply()
 *   -  getLogin(DataHeader hed, byte[] bUser)
 *   -  getUserlist(DataHeader hed, byte[] uList)
 *   -  examineUser(DataHeader hed, byte[] bUser)
 *   -  showMessage(DataHeader hed, byte[] contents)
 *   -  showCookie(DataHeader hed, byte[] bCookie)
 *   -  getStatus(String sLetter)
 *   -  getPrivs(int iPrivs)
 *   -  replyAFK(int userID)
 *   -  getDetails(DataHeader hed, byte[] buffer)
 */ 

import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.Vector;
import javax.swing.*;



public class Comms implements Constants
{
/*----------------------------------------------------------------------------}
{- quick function which checks the first DataHeader sent from the server contains
{- a C_HELLO
{----------------------------------------------------------------------------*/
public static int check_server(DataInputStream in)
    {
      DataHeader login_head = new DataHeader();
      try {
        login_head.recv(in);
        if (login_head.getCommand() == C_HELLO)
          System.out.println("Server acknowledgement (server version "+
                                                  login_head.getParam1()+"."+
                                                  login_head.getParam2()+"."+
                                                  login_head.getParam3()+").");
        else
          System.out.println("Received a \""+login_head.getCommand()+"\" from the server");
        return 0;
      }
      catch(Exception e) {
        System.out.println("Error receiving C_HELLO from the server.");
        return 1;
      }
    }

/*----------------------------------------------------------------------------}
{- dataReady is called each time a DataHeader is read from the server.
{- things should be received in the following order:
{----------------------------------------------------------------------------*/
public static synchronized void dataReady(DataHeader hed, byte[] buffer)
    {
      // this is a huge set of 'if' statements which might be better handled by a case/switch statement?
      switch(hed.getCommand()) {

      case C_HELLO:
        Cypher.DEBUG("C_HELLO received");
        break;

      case C_PING_REQ :
        Cypher.DEBUG("C_PING_REQ received - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
        //if (hed.getParam1()==0) Cypher.DEBUG("C_PING_REQ received from server");
        //else Cypher.DEBUG("C_PING_REQ received from "+Utils.IDToName(hed.getParam1()));
        doPingReply(hed);
        break;

      case C_PING_REPLY :
        Cypher.DEBUG("C_PING_REPLY received - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
        /*
        if (hed.getParam1()==0) {
          Cypher.DEBUG("C_PING_REPLY from server");
          errorFrame.showInf("Server pings OK","Reply from ping");
        }
        else {
          errorFrame.showInf(Utils.IDToName(hed.getParam1())+" pings OK","Reply from ping");
          Cypher.DEBUG("C_PING_REPLY from "+Utils.IDToName(hed.getParam1()));
        } */
        break;

      case C_LOGIN :
        getLogin(hed, buffer);
        break;

      case C_LOGOUT :
        try {
          // auto relogin for zaps?
          Cypher.DEBUG("C_LOGOUT received - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
          if (hed.getParam3()==P_LOGOUT_ZAP) {
            errorFrame.show("Someone has zapped you, you will now be logged out of Babel","ZAPPED!");
            Cypher.DEBUG("You were zapped! P1:P2 - "+hed.getParam1()+":"+hed.getParam2());
          }
          Input.logOut();
        }
        catch (Exception eClose) {
          Cypher.DEBUG("Error closing Cypher");
          Cypher.DEBUG(eClose);
        }
        break;

      case C_USERLIST :
        Cypher.DEBUG("C_USERLIST received");
        getUserlist(hed, buffer);
        break;
      
      case C_USER_CREATE :
        Cypher.DEBUG("C_USER_CREATE received");
        getDetails(hed, buffer);
        break;

      case C_USER_GET :
        Cypher.DEBUG("C_USER_GET received");
        examineUser(hed, buffer);
        break;

      case C_USER_PUT :
        Cypher.DEBUG("C_USER_PUT received");
        User xTarget = new User();
        xTarget = xTarget.translate(buffer);
        if (xTarget.Username.equals(Cypher.spod.Username)) {
          Cypher.spod.Popname = xTarget.Popname;
          Cypher.DEBUG("Your popname was updated.");
        }else Cypher.DEBUG("EEK! Details of "+xTarget.Username+" received!");
        break;

      case C_MSG_GET :
        Cypher.DEBUG("C_MSG_GET received");
        showMessage(hed, buffer);
        break;

      case C_SET_COOKIE :
        Cypher.DEBUG("C_SET_COOKIE received");
        showCookie(hed, buffer);
        break;

      case C_WATCH_LOGIN :
        Cypher.DEBUG("C_WATCH_LOGIN received - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
        checkWatchList(hed.getPacketSize(), buffer);
        break;

      case C_USERLIST_NEW :
        Cypher.DEBUG("C_USERLIST_NEW received");
        getUserlist(hed, buffer);
        break;

      case C_WEBSERV_LIST :
        Cypher.DEBUG("C_WEBSERV_LIST received - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
        getDetails(hed, buffer);
        break;

      default :
        Cypher.DEBUG("Unknown header: "+hed.getCommand()+" - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
        getDetails(hed, buffer);
        break;
      }
    } // end dataReady

  /*----------------------------------------------------------------------------}
  {- reply to a ping request. we can just change the header because incoming
  {- pings have the sender as Param1.
  {----------------------------------------------------------------------------*/
  public static void doPingReply(DataHeader hed)
  {
    try {
      hed.setCommand(C_PING_REPLY);
      hed.send(Cypher.out);    	
    }
    catch (IOException ePingReply) {
      Cypher.DEBUG("Error whilst doing ping reply");
      Cypher.DEBUG(ePingReply);
    }
  }


  /*----------------------------------------------------------------------------}
  {- server has sent back acceptance or denial of user login.
  {- call errorFrame directly
  {----------------------------------------------------------------------------*/
  public static void getLogin(DataHeader hed, byte[] bUser)
  {
    if (hed.getCommand()==C_LOGIN && hed.getParam3()==P_OK) {
      if (hed.getParam2()==P_OK_BUT_CLIENT_IS_OLD){
        errorFrame.showInf("Your client needs updating, it's out of date!", "OK Login");
      }
      else if (hed.getParam2()==P_OK_BUT_NO_WEB){
        errorFrame.showInf("Webserver is down at the moment.", "OK Login");
      }
      okLogin(hed, bUser);
    }
    else
    {
      Cypher.DEBUG("Login refused. Return params were - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
      if (hed.getParam3()==P_NLOGIN_BANNED){
        errorFrame.show("That username has been banned! You naughty person", "Error logging you in");
      }
      else if (hed.getParam3()==P_NLOGIN_BANNEDIP){
        errorFrame.show("Logins from your IP address have been banned.\nChange computer.", "Error logging you in");
      }
      else if (hed.getParam3()==P_NLOGIN_SERVERCLOSED){
        errorFrame.show("The server is not accepting any more connections.", "Error logging you in");
      }
      else if (hed.getParam3()==P_ALREADY_LOGGED_IN){
        errorFrame.show("You are already logged in!", "Error logging you in");
      }
      else {
        errorFrame.show("Undetermined login error - check username/password.", "Error logging you in");
        Cypher.DEBUG("Couldn't log user in, DataHeader was "+hed.getCommand()+" - "+hed.getParam1()+":"+hed.getParam2()+":"+hed.getParam3());
        Cypher.Login();
      }
    }
  }


  /*----------------------------------------------------------------------------}
  {- this is all utter crap.
  {----------------------------------------------------------------------------*/
  public static void getUserlist(DataHeader hed, byte[] bSpod)
  {
    int numUsers=0;
    int x = 0;
    try {
      numUsers=hed.getParam1();
      mainFrame.statusBar.setText(" CYPHER - There are currently "+numUsers+" users on Babel.");
      Cypher.userList.clear();
      Cypher.userNames.clear();

      while(x < numUsers) {
        NewSpod tSpod = new NewSpod();
        tSpod = tSpod.translate(bSpod, x);
        //System.out.println("User: "+tSpod.Username);
        //System.out.println("Privs: "+tSpod.Privs);
        Cypher.userList.add(x, tSpod);          // update all user details
        Cypher.userNames.add(x,tSpod.Username); // vector of just usernames for combo boxes.
        x++;
      }
      // do update here.
      mainFrame.listUsers.setUserlist(Cypher.userList);
      mainFrame.jList1.setListData(Cypher.userNames);
    }
    catch (Exception eUserlist) {
      Cypher.DEBUG("Error storing Userlist:");
      Cypher.DEBUG(eUserlist);
    }
  }


  /*----------------------------------------------------------------------------}
  {- what gets returned from doing an Examine...
  {- what gets sent back if the user doesn't exist/misspelt name?
  {- is it contained in an error code?
  {- should i rather return the xTarget to the GUI caller? is there much point?
  {- (other than good coding, obviously)
  {----------------------------------------------------------------------------*/
  public static void examineUser(DataHeader hed, byte[] bUser)
  {
    Color GREEN = new Color(33,99,33);
    Color RED = new Color(204,0,51);
    Cypher.DEBUG("Return from examine: P1: "+
               hed.getParam1()+" P2: "+hed.getParam2()+" P3: "+hed.getParam3());
    if (hed.getParam3()==P_USER_NEXIST) {
      errorFrame.show("That user does not exist. Check the spelling.","Error examining that user");
    }
    else
    {
      User xTarget = new User();
      xTarget = xTarget.translate(bUser);
      mainFrame.lUsername.setText(xTarget.Username);
      mainFrame.lRealName.setText(xTarget.RealName);
      mainFrame.lEmail.setText(xTarget.EmailAddress);
      mainFrame.lHomeCity.setText(xTarget.City);
      mainFrame.popnameField.setText(xTarget.Popname);

      // these are fucked. :)
      if ((xTarget.Privs & S_AUTHOR)==S_AUTHOR)
        Utils.setYes(mainFrame.lAuthor, GREEN);
      else
        Utils.setNo(mainFrame.lAuthor, RED);

      if ((xTarget.Privs & S_EXECBM)==S_EXECBM)
        Utils.setYes(mainFrame.lExecBM, GREEN);
      else
        Utils.setNo(mainFrame.lExecBM, RED);

      if ((xTarget.Privs & S_BM)==S_BM)
        Utils.setYes(mainFrame.lBM, GREEN);
      else
        Utils.setNo(mainFrame.lBM, RED);

      if ((xTarget.Privs & S_SECT_MAN)==S_SECT_MAN)
        Utils.setYes(mainFrame.lSectionOwner, GREEN);
      else
        Utils.setNo(mainFrame.lSectionOwner, RED);

      if ((xTarget.Privs & S_CC_POLICE)==S_CC_POLICE)
        Utils.setYes(mainFrame.lConfPolice, GREEN);
      else
        Utils.setNo(mainFrame.lConfPolice, RED);

      if ((xTarget.Privs & S_EXTERNAL_ACCESS)==S_EXTERNAL_ACCESS)
        Utils.setYes(mainFrame.lExternal, GREEN);
      else
        Utils.setNo(mainFrame.lExternal, RED);

      if ((xTarget.Privs & S_REGISTERED)==S_REGISTERED)
        Utils.setYes(mainFrame.lRegistered, GREEN);
      else
        Utils.setNo(mainFrame.lRegistered, RED);

      if ((xTarget.Privs & S_EXEC)==S_EXEC)
        Utils.setYes(mainFrame.lSheffield, GREEN);
      else
        Utils.setNo(mainFrame.lSheffield, RED);
    }
  }


  /*----------------------------------------------------------------------------}
  {- we've received a message, so tell us who it's from and display the message.
  {-
  {- Better to create a new receiveFrame with the user and message as constructor
  {- parameters? would probably be nicer. :)
  {----------------------------------------------------------------------------*/
  public static void showMessage(DataHeader hed, byte[] contents)
  {
    String message = "";
    String from = Utils.IDToName(hed.getParam1());

    try {
      DataInputStream mTemp = new DataInputStream(new ByteArrayInputStream(contents));
      message = Utils.makeString(mTemp, hed.getPacketSize());
    }
    catch (Exception eMakeString) {
      Cypher.DEBUG("Error showing incoming message");
	    Cypher.DEBUG(eMakeString);
    }

    if (mainFrame.oldSkoolRadio.isSelected()==true){
      // this should SO all be dealt with in the receiveFrame file. :)
      receiveFrame receive = new receiveFrame();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension receiveSize = receive.getSize();
      if (receiveSize.height > screenSize.height) receiveSize.height = screenSize.height;
      if (receiveSize.width > screenSize.width) receiveSize.width = screenSize.width;
      receive.setLocation((screenSize.width - receiveSize.width) / 2, (screenSize.height - receiveSize.height) / 2);
      receive.fromName.setText(from);
      receive.setTitle("Message from "+from);
      receive.messageArea.setText(message);
      receive.show();
    }else mainFrame.chatArea.append("<"+from+"> "+message+"\n\n");

    if (Cypher.AFK == 1){ replyAFK(hed.getParam1());}// send AFK message.
  }


  /*----------------------------------------------------------------------------}
  {- we've received the cookie. tell the user what it is. (error check)
  {----------------------------------------------------------------------------*/
  private static void showCookie(DataHeader hed, byte[] bCookie)
  {
    try
    {
      DataInputStream cTemp = new DataInputStream(new ByteArrayInputStream(bCookie));
      Cypher.spod.Cookie = Utils.makeString(cTemp, hed.getPacketSize());
      Cypher.DEBUG("Cookie was: "+Cypher.spod.Cookie);
    }
    catch (IOException eMakeString)
    {
  	  Cypher.DEBUG("Error displaying cookie.");
      Cypher.DEBUG(eMakeString);
    }
  }

  private static void checkWatchList(int size, byte[] bUser)
  {
    try {
      DataInputStream bTemp = new DataInputStream(new ByteArrayInputStream(bUser));
      String user = Utils.makeString(bTemp, size);
      for (int i=0;i<mainFrame.watchlistModel.size();i++) {
        if (user.equalsIgnoreCase((String)mainFrame.watchlistModel.elementAt(i)))
          errorFrame.showInf(user+" has just logged in","Watch List");
      }
    }
    catch (IOException eMakeString)
    {
      Cypher.DEBUG("problem decoding watch list malarky.");
    }
  }

  /*----------------------------------------------------------------------------}
  {- Login was OK, so start Cypher
  {----------------------------------------------------------------------------*/
  private static void okLogin(DataHeader hed, byte[] bUser)
  {
    Cypher.DEBUG("Login accepted.");
    Cypher.DEBUG("Your connection number is "+hed.getParam1());
    Cypher.DEBUG("Client status is " +hed.getParam2());
    Cypher.DEBUG("User status is " +hed.getParam3());
    Cypher.spod=Cypher.spod.translate(bUser);
    Cypher.spod.ConnectionNum = hed.getParam1();
/*    if ((Cypher.spod.Privs & S_INVIS_LOGIN)==S_INVIS_LOGIN)
    if ((Cypher.spod.Privs & S_ULTRAINVIS_LOGIN)==S_ULTRAINVIS_LOGIN) Cypher.spod.Ultra=true;*/
    new Cypher();
  }

  /*----------------------------------------------------------------------------}
  {- translate the status letter into something sensible. :)
  {----------------------------------------------------------------------------*/
  private static String getStatus(String sLetter)
  {
    if (sLetter.equals("A")) {return "Active Account";}
    if (sLetter.equals("B")) {return "Banned";}
    if (sLetter.equals("D")) {return "Account Deleted";}
    if (sLetter.equals("S")) {return "Second Account";}
    if (sLetter.equals("P")) {return "Pending";}
    if (sLetter.equals("W")) {return "Webserver";}
    if (sLetter.equals("Q")) {return "Waiting to be registered";}
    else return "Unknown Status field.";
  }


  /*----------------------------------------------------------------------------}
  {- translate the privs into something user-understandable.
  {-
  {- I'll finish writing this one day. :)
  {- this just needs to be for the examine box - check boxes? or YES/NO ?
  {----------------------------------------------------------------------------*/
  private static String getPrivs(int privs)
  {
    if (privs>=1073741824) {
      return "Author";
    }else if (privs>=536870912) {
      return "Exec BM/Sysop";
    }else if (privs>=268435456) {
      return "BM";
    }else if (privs>=2097152) {
      return "Conferencer Police";
    }else return "Normal spod";
  }

 /*----------------------------------------------------------------------------}
 {- we've received a message, but we're AFK, so reply to the user telling them so.
 {----------------------------------------------------------------------------*/
  private static void replyAFK(int userID)
  {
    char cUserID = (char)userID;
    // make this user-definable. easy to do.
    String afkMessage = "Sorry, but "+Cypher.spod.Username+" has buggered off for a bit. I'm sure they'll get back to you soon. Honest";  
    String afkPacket=cUserID + afkMessage;
    int afkMessagesize=afkPacket.length();
    DataHeader afkHeader = new DataHeader(C_MSG_SEND,1,0,0,afkMessagesize);
    try
    {
      afkHeader.send(Cypher.out);
      Cypher.out.writeBytes(afkPacket);
      Cypher.out.flush();    	
    }
    catch (IOException eAFK)
    {
      Cypher.DEBUG("Error whilst sending AFK message:");
      Cypher.DEBUG(eAFK);
    }
  }

 /*----------------------------------------------------------------------------}
 {- we've received a header that isn't dealt with, so get the information and log it.
 {----------------------------------------------------------------------------*/
  private static void getDetails(DataHeader hed, byte[] buffer)
  {
    DataInputStream pPacket = new DataInputStream(new ByteArrayInputStream(buffer));
    try {
      Cypher.DEBUG(Utils.makeString(pPacket, hed.getPacketSize()));
    } catch (IOException eMakeString) {
      Cypher.DEBUG("Error converting packet data to String");
      Cypher.DEBUG(eMakeString);
    }
  }

}